; bill
; Brad Smith, 2012
;
; Bill Murray test of automated image building

.define FAMITRACKER 1

; iNES header
.segment "HEADER"

INES_MAPPER = 0
INES_MIRROR = 0 ; 0 = horizontal mirroring, 1 = vertical mirroring
INES_SRAM   = 0 ; 1 = battery backed SRAM at $6000-7FFF

.byte 'N', 'E', 'S', $1A ; ID
.byte $02 ; 16k PRG bank count
.byte $01 ; 4k CHR bank count
.byte INES_MIRROR | (INES_SRAM << 1) | ((INES_MAPPER & $f) << 4)
.byte (INES_MAPPER & %11110000)
.byte $0, $0, $0, $0, $0, $0, $0, $0 ; padding

; CHR ROM
.segment "TILES"
.incbin "bill\pat0.bin"
.incbin "bill\pat1.bin"

; Vectors, defined in CODE segment.
.segment "VECTORS"
.word nmi
.word reset
.word irq

; zero page variables
.segment "ZEROPAGE"
temp_addr: .res 2

; RAM variables
.segment "BSS"

oam_ram = $0700

; CODE
.segment "CODE"

nmi:
    pha
    txa
    pha
    tya
    pha
    ; upload OAM
    lda #0
    sta $2003
    lda #>oam_ram
    sta $4014
    ; set scroll and turn on rendering
    lda #0
    sta $2005
    sta $2005
    lda #%00011110
    sta $2001
    ; play music
    .if FAMITRACKER
        jsr PLAY
    .endif
    pla
    tay
    pla
    tax
    pla
    rti

irq:
    rti

reset:
    ; standard startup
    sei       ; set interrupt flag (unnecessary, unless reset is called from code)
    cld       ; disable decimal mode
    ldx #$40
    stx $4017 ; disable APU IRQ
    ldx $ff
    txs       ; set up stack
    ldx #$00
    stx $2000 ; disable NMI
    stx $2001 ; disable render
    stx $4010 ; disable DPCM IRQ
    bit $2002 ; clear vblank flag
    ; wait for vblank
:
    bit $2002
    bpl :-
    ; clear memory
    lda #$00
    tax
:
    sta $0000, X
    sta $0100, X
    sta $0200, X
    sta $0300, X
    sta $0400, X
    sta $0500, X
    sta $0600, X
    sta $0700, X
    inx
    bne :-
    ; wait for second vblank
:
    bit $2002
    bpl :-
    ; PPU is now warmed up, NES is ready to go!
    .if FAMITRACKER
        jsr INIT ; start music
    .endif
    jsr reset_screen
    ; start the program
    lda #%10001000 ; NMI on, background pattern 0, sprite pattern 1
    sta $2000
main_loop:
    jmp main_loop

; image data
data_pal:
    .incbin "bill\pal0.bin"
    .incbin "bill\pal1.bin"
data_nmt:
    .incbin "bill\grid.bin"
    .incbin "bill\att.bin"
data_oam:
    .incbin "bill\oam.bin"

.macro latch_ppu addr
    lda $2002
    lda #>addr
    sta $2006
    lda #<addr
    sta $2006
.endmacro

reset_screen:
    ; load palette (32 bytes)
    latch_ppu $3F00
    ldx #0
:
    lda data_pal, X
    sta $2007
    inx
    cpx #32
    bne :-
    ; load nametable (1024 bytes)
    latch_ppu $2000
    ldx #4
    lda #<data_nmt
    sta temp_addr+0
    lda #>data_nmt
    sta temp_addr+1
:
    ldy #0
:
    lda (temp_addr), Y
    sta $2007
    iny
    bne :-
    inc temp_addr+1
    dex
    bne :--
    ; reset scroll    
    lda #0
    sta $2005
    sta $2005
    ; load oam (256 bytes)
    lda #0
    sta $2003
    ldx #0
:
    lda data_oam, X
    sta oam_ram, X
    inx
    bne :-
    ; done
    rts

.if FAMITRACKER

    .include "nsf_driver\driver.s"
    ;.incbin "bill_music.bin"
    .include "bill_music.asm"
            
    ;.segment "DPCM"
    ;    .incbin "bill_dpcm.bin"

.endif
    
; end of file
